unit SillyChatU;

interface

uses
  sysutils, SillyNetworkingU;

type

  TServerApp = class(TSharedObject)
  protected
    procedure ImproviseConsole;
    procedure CheckNewMessages;
    procedure SendTextMessage(s: string);
    procedure ProcessSendCommand(aCommand: string);
  public
    Server: TServer;
    procedure Run;
  end;

  TClientApp = class(TSharedObject)
  protected
    procedure ImproviseConsole;
    procedure CheckNewMessages;
    procedure SendTextMessage(s: string);
    procedure ProcessSendCommand(aCommand: string);
  public
    Client: TClient;
    procedure Run;
  end;

procedure RunServer;
procedure RunClient;

implementation

procedure WriteDebug(s: string);
begin
  WriteLN('SillyChat: ' + s);
end;

{ TClientApp }

procedure TClientApp.Run;
begin
  Client := TClient.Create;
  Client.Address := 'localhost';
  Client.Port := 30085;
  Client.Connect;
  ImproviseConsole;
end;

procedure TClientApp.ImproviseConsole;
var
  command: string;
begin
  while True do
  begin
    ReadLN(command);
    self.WriteDebug('Now executing: "' + command + '"');
    if command = 'exit' then
      break;
    if command = 'check' then
      CheckNewMessages;
    if Pos('send ', command) = 1 then
      ProcessSendCommand(command);
  end;
end;

procedure TClientApp.CheckNewMessages;
var
  message: TMemory;
  atLeast1messagePresent: Boolean;
begin
  atLeast1messagePresent := False;
  while True do
  begin
    message := Client.Inbox.Pop;
    if message <> nil then
    begin
      WriteDebug('Message arrived: ' + message.AsString);
      atLeast1messagePresent := True;
    end
    else
      break;
  end;
  if not atLeast1messagePresent then
    WriteDebug('No new messages.');
end;

procedure TClientApp.SendTextMessage(s: string);
var
  message: TMemory;
begin
  message := TMemory.Create;
  message.AsString := s;
  WriteDebug('Now sending "' + s + '"');
  Client.Outgoing.Push(message);
end;

procedure TClientApp.ProcessSendCommand(aCommand: string);
begin
  Delete(aCommand, 1, Length('send '));
  SendTextMessage(aCommand);
end;

{ TServerApp }

procedure TServerApp.Run;
begin
  Server := TServer.Create;
  Server.Port := 30085;
  Server.Start;
  ImproviseConsole;
end;

procedure TServerApp.ImproviseConsole;
var
  command: string;
begin
  command := '';
  while True do
  begin
    ReadLN(command);
    WriteDebug('Now executing: "' + command + '"');
    if command = 'exit' then
      break;
    if command = 'check' then
      CheckNewMessages;
    if Pos('send ', command) = 1 then
      ProcessSendCommand(command);
  end;
end;

procedure TServerApp.CheckNewMessages;
var
  clients: TClientDynArray;
  client: TClient;
  message: TMemory;
begin
  clients := Server.GetClients;
  WriteDebug('Now checking for incoming messages; number of clients: ' + IntToStr(Length(clients)));
  for client in clients do
  begin
    while True do
    begin
      message := client.Inbox.Pop;
      if message <> nil then
        self.WriteDebug('Message arrived: "' + message.AsString + '"')
      else
        break;
    end;
  end;
end;

procedure TServerApp.SendTextMessage(s: string);
var
  clients: TClientDynArray;
  client: TClient;
  message: TMemory;
begin
  clients := Server.GetClients;
  WriteDebug('Now sending message "' + s + '" to clients; '
    + 'number of clients: ' + IntToStr(Length(clients)));
  message := TMemory.Create;
  message.AsString := s;
  for client in clients do
    client.Outgoing.Push(message);
end;

procedure TServerApp.ProcessSendCommand(aCommand: string);
begin
  Delete(aCommand, 1, Length('send '));
  SendTextMessage(aCommand);
end;

{ TNoClass }

procedure RunServer;
var
  app: TServerApp;
begin
  app := TServerApp.Create;
  app.Run;
end;

procedure RunClient;
var
  app: TClientApp;
begin
  app := TClientApp.Create;
  app.Run;
end;


end.

